package kernel.service;

import kernel.log.Log;
import kernel.binstore.BinStore;
import lib.KVStore;

using tink.CoreApi;

class ServiceManager {
	private static final services:Map<String, Service> = new Map();

	@:allow(kernel.Init)
	private static function init() {
		startAllEnabled();
	}

	/**
		Add a service to be automatically started.
	**/
	public static function enable(name:String):Outcome<Noise, String> {
		if (!services.exists(name)) {
			return Failure("Service must be started before enable");
		}

		var store = KVStore.getStoreForClass();
		store.load();

		var enabled = store.get("enabled", []);
		enabled.push(name);
		store.set("enabled", enabled);

		store.save();

		return Success(Noise);
	}

	/**
		Remove a service from being automatically started.
	**/
	private static function disable(name:String):Outcome<Noise, String> {
		var store = KVStore.getStoreForClass();
		store.load();

		var enabled:Array<String> = store.get("enabled");
		var index = enabled.indexOf(name);
		if (index == -1) {
			return Failure("Service not found");
		}

		enabled.splice(index, 1);
		store.save();

		return Success(Noise);
	}

	private static function startAllEnabled() {
		var store = KVStore.getStoreForClass();
		store.load();

		var enabled:Array<String> = store.get("enabled", []);
		for (name in enabled) {
			start(name);
		}
	}

	private static function load(name:String):Null<Service> {
		var store = new KVStore('service/${name}');
		store.load();
		if (!store.exists("service")) {
			return null;
		}

		return store.get("service");
	}

	public static function register(name:String, binName:String, args:Array<String>):Outcome<Noise, String> {
		if (BinStore.getNameByAlias(binName) == null) {
			return Failure("bin not found");
		}

		if (load(name) != null) {
			return Failure("service already exists");
		}

		var service = new Service(binName, name, args);

		var store = new KVStore('service/${name}');
		store.set("service", service);
		store.save();

		Log.info('Service ${name} registered');
		return Success(Noise);
	}

	public static function unregister(name:String):Outcome<Noise, String> {
		if (services.exists(name)) {
			return Failure("service is running");
		}

		KVStore.removeNamespace('service/${name}');
		Log.info('Service ${name} unregistered');
		return Success(Noise);
	}

	public static function start(name:String):Outcome<Noise, String> {
		var service = load(name);
		if (service == null) {
			return Failure("service not found");
		}

		service.start();
		services.set(name, service);

		Log.info('Service ${name} started');
		return Success(Noise);
	}

	public static function stop(name:String):Outcome<Noise, String> {
		if (!services.exists(name)) {
			return Failure("service not found");
		}

		var service = services.get(name);
		service.stop();
		services.remove(name);

		Log.info('Service ${name} stopped');
		return Success(Noise);
	}

	public static function listRunning():Array<String> {
		var running = [];
		for (name in services.keys()) {
			running.push(name);
		}
		return running;
	}

	public static function get(name:String):Null<Dynamic> {
		if (!services.exists(name)) {
			return null;
		}

		// TODO: Maybe there is a way to check types here?

		var srv = services.get(name);
		return srv.ps;
	}
}
